#include <stdio.h>
#include <stdlib.h>
#include "Historico.h"

/* Função que cria um novo histórico. */
Historico criarHistorico(int tamanho) {
	int i;
	
	// Cria a estrutura de dados.
	Historico historico = (Historico)malloc(sizeof(struct THistorico));
	historico->tamanho = tamanho;
	
	// Aloca o vetor circular, zera seus elementos e o marcador da próxima posição.
	historico->vetor = (Livro *)malloc(tamanho * sizeof(Livro));
	for (i = 0; i < tamanho; i++) historico->vetor[i] = NULL;
	historico->prox = 0;
	
	return historico;
}

/* Função que adiciona um livro ao histórico. */
void adicionarAoHistorico(Historico historico, Livro livro) {
	// Armazena o livro na próxima posição a ser ocupada.
	historico->vetor[historico->prox] = livro;
	
	// Passa para a próxima posição, considerando o vetor circular
	// (ou seja, quando chegar ao final volta ao início).
	historico->prox = (historico->prox + 1) % historico->tamanho;
}

/* Função que imprime o histórico. */
void imprimirHistorico(Historico historico) {
	// Começa a impressão a partir da posição do próximo elemento a ser ocupado.
	int i, pos = historico->prox;
	
	// Imprime o cabeçalho.
	printf("Histórico:\n");
	
	// Verifica se o histórico está vazio.
	if (pos == 0 && historico->vetor[pos] == NULL)
		printf("(vazio)\n");
	
	// Caso contrário, imprime os livros.
	else for (i = 0; i < historico->tamanho; i++) {
		Livro livro = historico->vetor[pos];

		// Verifica se é nula caso o vetor não esteja cheio.
		if (livro != NULL)
			printf("- %s\n", livro->titulo);
		
		// Passa para a próxima posição, considerando o vetor circular.
		pos = (pos + 1) % historico->tamanho;
	}
}

/* Função que destrói o histórico. */
void destruirHistorico(Historico historico) {
	free(historico->vetor);
	free(historico);
}
